SystemOrganization addCategory: #'Pier-ScriptingAPI'!
SystemOrganization addCategory: #'Pier-ScriptingAPI-Scalability'!

!Character methodsFor: '*pier-scriptingapi' stamp: 'kph 9/17/2006 02:33'!
join: aCollection

	^ String streamContents: [ :stream |
		aCollection
			do: [ :each | stream nextPutAll: each printString ]
			separatedBy: [ stream nextPut: self ] ]! !

!String methodsFor: '*pier-scriptingapi' stamp: 'kph 9/17/2006 02:34'!
join: aCollection
 
	^ String streamContents: [ :stream |
		aCollection
			do: [ :each | stream nextPutAll: each ]
			separatedBy: [ stream nextPutAll: self ] ]! !

Object subclass: #PRModelScripter
	instanceVariableNames: 'kernel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ScriptingAPI'!

!PRModelScripter commentStamp: 'kph 9/17/2006 01:56' prior: 0!
Implementing the same protocol as PRCommandScripter. PRModelScripter is simple programmatic interface to Pier which operates on the model directly.

If you wish to use PRModelScripter to load a Magma persisted model then you have to provide the MagmaSession, manage transactions, and explicitly instanciate PRModelScripter on the persisted kernel instance obtained from '(aMagmaSession root store at: 'pier') kernel'. see #exampleUsage.!

PRModelScripter subclass: #PRCommandScripter
	instanceVariableNames: 'logging'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ScriptingAPI'!

!PRCommandScripter commentStamp: '<historical>' prior: 0!
PRCommandScripter a simple programmatic interface to Pier, which uses the PRCommand mechanism to interface with the model. 

This means that all actions may be logged as if made through the Seaside interface. 

Logging can be turned on/off with #noLog #yesLog. If logging is disabled then the PRCommandScripter should be equivalent to PRModelScriptor but with an implementation more consistent with the end user experience (i.e. validation provided by PRCommands etc.). 

PRModelScriptor provides examples of how the Model may be used but PRCommandScriptor is the preferred interface to command functions. 

Note: Care is taken in the the PRCommandScripter API to explicitly ensure that an instance keeps no references to the model, or commands onto the model, in local instance varsiable. This is so as not to become a potential garbage collection inhibiting 'hard ref' to the model.
!

PRCommandScripter subclass: #PRCommandAppScripter
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ScriptingAPI'!

!PRCommandAppScripter commentStamp: '<historical>' prior: 0!
PRCommandAppScripter provides a scripting interface which executes its commands in the context of a registered seaside application.

The aim of this interface is to ensure that 'WACurrentSession value' returns a useful result when running scripts. PRMagmaPersistency uses 'WACurrentSession value', and command actions should result in the persistent store being updated just as when executed through the seaside user interface.

usage:

PRCommandAppScripter kernelNamed: 'pier'  
searches for the first application whose configuration entry #kernelName matches the requested name. A session is created using the configuration settings of this application. 

caveat: if you have more than one application serving the same pier kernel with different preferences you have been warned. If no application is found, it will fallback to use any application that is serving all kernels (#mainClass is set to PRMainAllKernels). If you have more than one of these serving more than one repository then it will not check to ensure that it has the correct repository.

PRCommandAppScripter onApp: 'st'
explicitly gives entry point for the application in whose context the scripts are to be run. It is assumed that the value of #kernelName is valid. For an application using PRMainAllKernels, use #onApp:onKernelName: to explicitly target the required kernel.

!

!PRCommandAppScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/18/2006 02:54'!
findAppServingAllKernels

^WADispatcher default allApplications detect: [ :app |
					(app preferenceAt: #mainClass) respondsTo: #isServingAllKernels ]
			 ! !

!PRCommandAppScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/18/2006 02:54'!
findAppServingKernel: requestedKernelName ifNone: notThereBlock

^WADispatcher default allApplications detect: [ :app |
					(app preferenceAt: #kernelName) = requestedKernelName ] 
	ifNone: notThereBlock ! !

!PRCommandAppScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 17:38'!
inSession: aSession getKernel: kernelName

| |
^(PRKernel instanceNamed: kernelName) persistency resolveKernelFor: aSession. 
 ! !

!PRCommandAppScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/18/2006 02:53'!
onApp: entryPointName

| application sessionClass kernelName |

application :=  WADispatcher default entryPointAt: entryPointName.
sessionClass := application preferenceAt: #sessionClass.
kernelName := application preferenceAt: #kernelName.

^self setKernel: (PRKernel instanceNamed: kernelName);
	 setSession: (sessionClass application: application); 
	yourself
! !

!PRCommandAppScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/18/2006 02:54'!
onApp: entryPointName kernelNamed: kernelName

| application sessionClass  |

application :=  WADispatcher default entryPointAt: entryPointName.
sessionClass := application preferenceAt: #sessionClass.

^self setKernel: (PRKernel instanceNamed: kernelName);
	 setSession: (sessionClass application: application); 
	yourself
! !

!PRCommandAppScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/18/2006 02:54'!
onKernelNamed: requestedKernelName

"find a registered seaside application serving the desired kernel and make a session using the settings of that application"

| application sessionClass |

application :=  self findAppServingKernel: requestedKernelName 
					ifNone: [ self findAppServingAllKernels ].

sessionClass := application preferenceAt: #sessionClass.

^self new setKernel: (PRKernel instanceNamed: requestedKernelName);
	 setSession: (sessionClass application: application); 
	yourself
! !

!PRCommandAppScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/16/2006 05:06'!
executeCommand: aCommand

WACurrentSession use: session during: [ super executeCommand: aCommand ].! !

!PRCommandAppScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/16/2006 03:59'!
freshSession

	self setSession: ( self session class application: (self session application) ).! !

!PRCommandAppScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 17:40'!
kernel

^ kernel persistency resolveKernelFor: self . ! !

!PRCommandAppScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 05:08'!
session

^session! !

!PRCommandAppScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 02:40'!
setSession: aSession

session := aSession! !

!PRCommandAppScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/17/2006 04:46'!
transaction: aBlock

self session magmaSession stubOut: aBlock value! !

!PRCommandScripter methodsFor: 'public api' stamp: 'kph 9/24/2006 00:59'!
addPageAt: aPath name: aPageName

| aCommand |

"returns the new page added"

aCommand := PRAddCommand context: (self contextOn: aPath).
aCommand name: aPageName.
 
self executeCommand: aCommand.

^aCommand context structure childAt: aCommand name

  ! !

!PRCommandScripter methodsFor: 'public api' stamp: 'kph 9/16/2006 05:05'!
addPageAt: aPath name: aName contents: someContent

| newPage aCommand |

newPage := self addPageAt: aPath name: aName.

aCommand := PREditCommand context: (self contextOn: newPage).
aCommand 	write: someContent
			using: PRPage descriptionContents.

self executeCommand: aCommand.

^newPage! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/16/2006 01:14'!
contextOn: aPathOrStructure

^ PRContext kernel: self kernel structure: (self structureAt: aPathOrStructure) ! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/16/2006 05:03'!
executeCommandWithoutLogging: aCommand

"code stolen from PRCommand and logging is commented out"

	aCommand isView
		ifTrue: [ aCommand shouldNotImplement ].
	aCommand kernel persistency doing: command critical: [
		aCommand doValidate; doExecute.
		"aCommand isLogged
			ifTrue: [ aCommand doPersistency ]" ].
	aCommand doAnswer! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/16/2006 05:03'!
executeCommand: aCommand.

self isLogging ifTrue: [ aCommand execute ] ifFalse: [ self executeCommandWithoutLogging: aCommand ].! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:24'!
isLogging

^logging ifNil: [ false ]! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 01:44'!
noLog

logging := false! !

!PRCommandScripter methodsFor: 'public api' stamp: 'kph 9/17/2006 02:18'!
removePageAt: aPath
 
self executeCommand: (PRRemoveCommand context: (self contextOn: aPath))
! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 01:44'!
yesLog

logging := true! !

!PRModelScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/17/2006 02:04'!
exampleUsage

| magmaSession scripter |

magmaSession := MagmaSession openLocal: 'magma-pier'.
magmaSession connectAs: 'model-scripter'.

scripter := PRModelScripter onKernel: (magmaSession root kernelAt: 'Squeak') kernel.

magmaSession commit: [

	scripter addPageAt: '/' name: 'Testing PRModelScripter' contents: 'testing PRModelScripter... ok'.

].

magmaSession disconnectAndClose.! !

!PRModelScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/14/2006 23:52'!
onKernelNamed: aKernelName
 
^self onKernel: (PRKernel instanceNamed: aKernelName).! !

!PRModelScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/14/2006 23:52'!
onKernel: aKernel
 
^self new setKernel: aKernel ; yourself! !

!PRModelScripter methodsFor: 'public api' stamp: 'kph 9/16/2006 01:10'!
addPageAt: aPath name: aName

^(self structureAt: aPath) addChild: (PRPage named: aName)! !

!PRModelScripter methodsFor: 'public api' stamp: 'kph 9/16/2006 01:12'!
addPageAt: aPath name: name contents: someContent

^ (self addPageAt: aPath name: name) contents: someContent! !

!PRModelScripter methodsFor: 'compatibility' stamp: 'kph 9/15/2006 02:11'!
isLogging

^false! !

!PRModelScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 17:40'!
kernel

^ kernel! !

!PRModelScripter methodsFor: 'compatibility' stamp: 'kph 9/15/2006 02:11'!
noLog! !

!PRModelScripter methodsFor: 'public api' stamp: 'kph 9/16/2006 01:20'!
removePageAt: aPath

 | aPage |

aPage := self structureAt: aPath.

aPage isRoot
		ifTrue: [ self error: 'Unable to remove root node.' ].

PRIncomingReferences
		from: self kernel root
		to: (aPage enumerator
			with; all; hidden;
			contents)
		do: [ :each | each target: nil ].

^ aPage remove! !

!PRModelScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 02:41'!
setKernel: aKernel

kernel := aKernel
! !

!PRModelScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/16/2006 00:47'!
structureAt: aPathOrStructure

^aPathOrStructure isString ifTrue: [ PRPathLookup start: self kernel root path: aPathOrStructure ]
			   			  ifFalse: [ aPathOrStructure ].! !

!PRModelScripter methodsFor: 'public api' stamp: 'kph 9/17/2006 04:45'!
transaction: aBlock

^aBlock value! !

!PRModelScripter methodsFor: 'compatibility' stamp: 'kph 9/15/2006 02:11'!
yesLog! !

Object subclass: #PRPierifyToPage
	instanceVariableNames: 'target kernel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ScriptingAPI'!

!PRPierifyToPage commentStamp: '<historical>' prior: 0!
Abstract class of classes designed to load data from a datasource #target: into a pier page
Can be used with any of the Scripter interface classes
See subclasses for usage examples!

PRPierifyToPage subclass: #PRPierifySmalltalkClass
	instanceVariableNames: ''
	classVariableNames: 'Pace'
	poolDictionaries: ''
	category: 'Pier-ScriptingAPI-Scalability'!

!PRPierifySmalltalkClass commentStamp: '<historical>' prior: 0!
Example of how to use the scripting interface to import data. This class is instanciated with the data source as its target.

(PRPierifySmalltalkClass target: ProtoObject) into: (PRCommandScripter onKernelNamed: 'Squeak' ).
(PRModelScripter onKernelNamed: 'Squeak') removePageAt: 'ProtoObject'.

To create a sizeable database see class methods #example...

!

!PRPierifySmalltalkClass class methodsFor: 'as yet unclassified' stamp: 'kph 9/16/2006 03:30'!
exampleAllClasses

| model count |

count := 0.
model := PRCommandAppScripter onKernelNamed: 'Squeak'.

Transcript show: model session magmaSession location; cr.
model noLog. 

ProtoObject allSubclassesDo: [ :theClass | 

count := count + 1.
Transcript show: count printString, ': ', theClass name; cr.
[
 
 (PRPierifySmalltalkClass target: theClass) into: model.
 (Delay forSeconds: self pace) wait.
] 
on: PRConflictingNameError do: [ :e | Transcript show: theClass name, ' already exists'; cr. ].


].

 
"
self pace: 30.
[ self exampleAllClasses ] forkAt: 15"

! !

!PRPierifySmalltalkClass class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:35'!
exampleProtoObject

| model |

model := PRCommandAppScripter onKernelNamed: 'Squeak'.
model noLog. 

ProtoObject in: [ :theClass | 

Transcript show: theClass name; cr.
(PRPierifySmalltalkClass target: theClass) into: model ].

"self exampleProtoObject"

! !

!PRPierifySmalltalkClass class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:38'!
pace

^Pace ifNil: [ 20 ].! !

!PRPierifySmalltalkClass class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:38'!
pace: t

Pace := t! !

!PRPierifySmalltalkClass class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 01:17'!
target: aClass  

^self new setTarget: aClass ; yourself! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/15/2006 03:43'!
classDescriptionToWiki: stream

stream nextPutAll: '!!Description'; cr.
stream nextPutAll: target class comment ; cr.
 

! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/15/2006 00:13'!
classSubclassesToWiki: stream.

target subclassesDo: [ :sc | stream nextPutAll: '- *', sc name, '*'; cr ]
  ! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/14/2006 23:56'!
methodSelectors: methodDictionary toWiki: stream

methodDictionary keysDo: [ :methodSelector | stream nextPutAll: '- *', methodSelector, '*'; cr ]
 
! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/15/2006 02:15'!
pageContentsOn: stream  

self classDescriptionToWiki: stream.

stream nextPut: $_; cr.

stream nextPutAll: '!!!!!!Subclasses'; cr.

self classSubclassesToWiki: stream.

stream nextPut: $_; cr.

stream nextPutAll: '!!!!!!Instance Methods'; cr.

self methodSelectors: target methodDictionary toWiki: stream.

stream nextPutAll: '!!!!!!Class Methods'; cr.

self methodSelectors: target class methodDictionary toWiki: stream.
 ! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/15/2006 04:31'!
pageName

^target name asString

 ! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/15/2006 16:44'!
parentPath

^ ( target allSuperclasses reverse) joinWith: $/
 
"
(self target: ProtoObject) parentPath
(self target: String) parentPath
"! !

!PRPierifyToPage methodsFor: 'as yet unclassified' stamp: 'kph 9/17/2006 04:44'!
into: aScripter
 
aScripter transaction: [ aScripter addPageAt: (self parentPath) name: (self pageName) contents: (self pageContents) ].


! !

!PRPierifyToPage methodsFor: 'page items' stamp: 'kph 9/15/2006 00:09'!
pageContents
	^ String streamContents: [ :stream | self pageContentsOn: stream ].! !

!PRPierifyToPage methodsFor: 'page items' stamp: 'kph 9/15/2006 00:11'!
pageContentsOn: stream

^ self subclassResponsibility! !

!PRPierifyToPage methodsFor: 'page items' stamp: 'kph 9/15/2006 00:11'!
pageName

^ self subclassResponsibility! !

!PRPierifyToPage methodsFor: 'page items' stamp: 'kph 9/15/2006 00:12'!
parentPath

^ self subclassResponsibility! !

!PRPierifyToPage methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 00:04'!
setTarget: aDataSource

target := aDataSource! !

!SequenceableCollection methodsFor: '*pier-scriptingapi' stamp: 'kph 9/15/2006 15:43'!
joinTokens: aString
    ^ String streamContents:
        [:s |
        self do: [:ea | s nextPutAll: ea asString] separatedBy: [s nextPutAll: aString]] ! !

!SequenceableCollection methodsFor: '*pier-scriptingapi' stamp: 'kph 9/17/2006 02:33'!
join: aCollection

	^ self class streamContents: [ :stream |
		aCollection
			do: [ :each | stream nextPut: each ]
			separatedBy: [ stream nextPutAll: self ] ]! !

!Collection methodsFor: '*pier-scriptingapi' stamp: 'kph 9/15/2006 00:36'!
joinWith: separator
	
^separator join: self ! !

!WADispatcher methodsFor: '*pier-scriptingapi' stamp: 'kph 9/15/2006 04:03'!
allApplications

| apps |

 apps := self entryPoints select: [ :ep | ep isApplication ].

self entryPoints select: [ :ep | ep isDispatcher ] thenDo: [:ep | apps addAll: ep allApplications ].

^apps

 ! !
